require "clerk/version"
require "clerk/api"
require "clerk/engine"
require "clerk/id"
require "clerk/helpers"
require "faraday"

module Clerk
  class << self
    attr_reader :config

    def configure
      @config = Configuration.new
      yield config
    end

    def database_connection_url
      @database_connection_url ||= "#{Clerk.config.database_url}?prepared_statements=false"
    end

    def api
      @conn ||= Clerk::Api::Connection.new(
        (ENV["CLERK_API_PATH"] || "https://api.clerk.dev"),
        key_secret
      )
    end

    def key
      ENV['CLERK_KEY']
    end

    def key_secret
      ENV['CLERK_KEY']&.slice(5..-1)
    end

    def accounts_url
      @accounts_url ||= "https://#{Clerk.config.accounts_host}"
    end

    def app_url
      @app_url ||= "https://#{Clerk.config.app_host}"
    end

    def cipher_key
      @cipher_key ||= ::Base64.strict_decode64(Clerk.config.cipher_key)
    end

    def decrypt(encrypted_message)
      cipher = OpenSSL::Cipher.new("aes-256-gcm")

      encrypted_data, iv, auth_tag = encrypted_message.split("--".freeze).map { |v| ::Base64.strict_decode64(v) }

      # Currently the OpenSSL bindings do not raise an error if auth_tag is
      # truncated, which would allow an attacker to easily forge it. See
      # https://github.com/ruby/openssl/issues/63
      raise InvalidMessage if (auth_tag.nil? || auth_tag.bytes.length != 16)

      cipher.decrypt
      cipher.key = cipher_key
      cipher.iv  = iv
      cipher.auth_tag = auth_tag
      cipher.auth_data = ""

      message = cipher.update(encrypted_data)
      message << cipher.final
      message
    end

    def email(
      account: nil,
      to_email_address: nil,
      from_email_name: nil,
      email_template_token: nil,
      replacements: nil,
      subject: nil,
      body: nil
    )
      email = Clerk::Email.create(
        account_id: account&.id,
        to_email_address: to_email_address, 
        from_email_name: from_email_name,
        email_template_token: email_template_token,
        replacements: replacements.nil? ? nil : replacements.to_json,
        subject: subject,
        body: body
      )
    end  
  end

  class Configuration
    attr_accessor :accounts_host,
      :app_host,
      :host,
      :cipher_key,
      :database_url, 
      :email_address_verification,      
      :ngrok_authtoken, 
      :tunnel_cert, 
      :tunnel_key
  end
end
